@@ -35,46 +35,49 @@ using System.Collections.Generic;
 using Hyena;
 using Hyena.Data;
 using Hyena.Data.Sqlite;
+using Hyena.Query;
 
 using Banshee.Collection;
 using Banshee.Database;
 
 namespace Banshee.Collection.Database
-{   
-    public abstract class DatabaseFilterListModel<T, U> : FilterListModel<U>, ICacheableDatabaseModel
+{
+    public abstract class DatabaseFilterListModel<T, U> : FilterListModel<U>, ICacheableDatabaseModel, ISearchable
         where T : U, new () where U : ICacheableItem, new()
     {
         private readonly BansheeModelCache<T> cache;
         private readonly Banshee.Sources.DatabaseSource source;
-        
+
         private long count;
         private string reload_fragment;
-        
+
         private string reload_fragment_format;
         protected string ReloadFragmentFormat {
             get { return reload_fragment_format; }
             set { reload_fragment_format = value; }
         }
-        
-        protected readonly U select_all_item;
 
-        public DatabaseFilterListModel (string name, string label, Banshee.Sources.DatabaseSource source, 
-                                        DatabaseTrackListModel trackModel, HyenaSqliteConnection connection, SqliteModelProvider<T> provider, U selectAllItem, string uuid) 
+        protected readonly U select_all_item;
+        private HyenaSqliteConnection connection;
+
+        public DatabaseFilterListModel (string name, string label, Banshee.Sources.DatabaseSource source,
+                                        DatabaseTrackListModel trackModel, HyenaSqliteConnection connection, SqliteModelProvider<T> provider, U selectAllItem, string uuid)
             : base (trackModel)
         {
             this.source = source;
             FilterName = name;
             FilterLabel = label;
             select_all_item = selectAllItem;
-            
+
+            this.connection = connection;
             cache = new BansheeModelCache <T> (connection, uuid, this, provider);
             cache.HasSelectAllItem = true;
         }
-        
+
         public override void Clear ()
         {
         }
-        
+
         protected virtual void GenerateReloadFragment ()
         {
             ReloadFragment = String.Format (
@@ -114,23 +117,23 @@ namespace Banshee.Collection.Database
                 }
             }
         }
-        
+
         public virtual bool CachesValues { get { return false; } }
-        
+
         protected abstract string ItemToFilterValue (object o);
 
         // Ick, duplicated from DatabaseTrackListModel
         public override string GetSqlFilter ()
         {
             string filter = null;
-            
+
             // If the only item is the "All" item, then we shouldn't allow any matches, so insert an always-false condition
             if (Count == 1) {
                 return "0=1";
             } else {
                 ModelHelper.BuildIdFilter<object> (GetSelectedObjects (), FilterColumn, null,
                     delegate (object item) {
-                        if (item != select_all_item) {
+                        if (!select_all_item.Equals (item)) {
                             return ItemToFilterValue (item);
                         }
                         return null;
@@ -138,39 +141,45 @@ namespace Banshee.Collection.Database
                     delegate (string new_filter) { filter = new_filter; }
                 );
             }
-            
+
             return filter;
         }
-        
+
         public abstract void UpdateSelectAllItem (long count);
 
         public override void Reload (bool notify)
         {
             GenerateReloadFragment ();
 
-            cache.SaveSelection ();
-            cache.Reload ();
-            cache.UpdateAggregates ();
-            cache.RestoreSelection ();
+            lock (cache) {
+                connection.BeginTransaction ();
+                cache.SaveSelection ();
+                cache.Reload ();
+                cache.UpdateAggregates ();
+                cache.RestoreSelection ();
+                connection.CommitTransaction ();
+
+                count = cache.Count + 1;
+            }
 
-            count = cache.Count + 1;
-            
             UpdateSelectAllItem (count - 1);
 
             if (notify)
                 OnReloaded ();
         }
-        
+
         public override U this[int index] {
             get {
                 if (index == 0)
                     return select_all_item;
 
-                return cache.GetValue (index - 1);
+                lock (cache) {
+                    return cache.GetValue (index - 1);
+                }
             }
         }
 
-        public override int Count { 
+        public override int Count {
             get { return (int) count; }
         }
 
@@ -187,9 +196,13 @@ namespace Banshee.Collection.Database
         }
 
         public int CacheId {
-            get { return (int) cache.CacheId; }
+            get {
+                lock (cache) {
+                    return (int) cache.CacheId;
+                }
+            }
         }
-        
+
         public IEnumerable<object> GetSelectedObjects ()
         {
             foreach (object o in SelectedItems) {
@@ -199,12 +212,32 @@ namespace Banshee.Collection.Database
 
         public override void InvalidateCache (bool notify)
         {
-            cache.Clear ();
+            lock (cache) {
+                cache.Clear ();
+            }
             if (notify) {
                 OnReloaded ();
             }
         }
-        
+
+        private QueryFieldSet query_fields;
+        public QueryFieldSet QueryFields {
+            get { return query_fields; }
+            protected set { query_fields = value; }
+        }
+
+        public int IndexOf (QueryNode query, long offset)
+        {
+            lock (cache) {
+                if (query == null) {
+                    return -1;
+                }
+
+                int index = (int) cache.IndexOf (query.ToSql (QueryFields), offset);
+                return index >= 0 ? index + 1 : index;
+            }
+        }
+
         public abstract string FilterColumn { get; }
 
         public virtual string JoinTable { get { return null; } }
