@@ -43,6 +43,7 @@ using Banshee.Metadata;
 using Banshee.Preferences;
 using Banshee.Query;
 using Banshee.Sources;
+using Banshee.Library;
 using Banshee.ServiceStack;
 using Banshee.Streaming;
 
@@ -156,6 +157,12 @@ namespace Banshee.Collection.Database
             Save (NotifySaved);
         }
 
+        public override void UpdateLastPlayed ()
+        {
+            base.UpdateLastPlayed ();
+            Save (NotifySaved, BansheeQuery.LastPlayedField);
+        }
+
         public void Save (bool notify, params QueryField [] fields_changed)
         {
             // If either the artist or album changed,
@@ -539,6 +546,18 @@ namespace Banshee.Collection.Database
             set { base.BitRate = value; }
         }
 
+        [DatabaseColumn]
+        public override int SampleRate {
+            get { return base.SampleRate; }
+            set { base.SampleRate = value; }
+        }
+
+        [DatabaseColumn]
+        public override int BitsPerSample {
+            get { return base.BitsPerSample; }
+            set { base.BitsPerSample = value; }
+        }
+
         [DatabaseColumn("Rating")]
         protected int rating;
         public override int Rating {
@@ -629,6 +648,14 @@ namespace Banshee.Collection.Database
             }
         }
 
+        public PathPattern PathPattern {
+            get {
+                var src = PrimarySource;
+                var pattern = src == null ? null : src.PathPattern;
+                return pattern ?? MusicLibrarySource.MusicFileNamePattern;
+            }
+        }
+
         public bool CopyToLibraryIfAppropriate (bool force_copy)
         {
             bool copy_success = true;
@@ -639,10 +666,10 @@ namespace Banshee.Collection.Database
                 return copy_success;
             }
 
-            bool in_library = old_uri.AbsolutePath.StartsWith (PrimarySource.BaseDirectoryWithSeparator);
+            bool in_library = old_uri.IsLocalPath ? old_uri.AbsolutePath.StartsWith (PrimarySource.BaseDirectoryWithSeparator) : false;
 
             if (!in_library && (LibrarySchema.CopyOnImport.Get () || force_copy)) {
-                string new_filename = FileNamePattern.BuildFull (PrimarySource.BaseDirectory, this, Path.GetExtension (old_uri.ToString ()));
+                string new_filename = PathPattern.BuildFull (PrimarySource.BaseDirectory, this, Path.GetExtension (old_uri.ToString ()));
                 SafeUri new_uri = new SafeUri (new_filename);
 
                 try {
