@@ -1,4 +1,4 @@
-// 
+//
 // ColumnCellDateTime.cs
 //
 // Author:
@@ -32,7 +32,7 @@ using Hyena.Data.Gui;
 
 namespace Banshee.Collection.Gui
 {
-    public enum DateTimeFormat 
+    public enum DateTimeFormat
     {
         Long,
         ShortDate,
@@ -40,40 +40,40 @@ namespace Banshee.Collection.Gui
         ShortTime,
         LongTime
     }
-        
+
     public class ColumnCellDateTime : ColumnCellText
     {
         public ColumnCellDateTime (string property, bool expand) : base (property, expand)
         {
         }
-        
+
         private DateTimeFormat format = DateTimeFormat.Long;
         public DateTimeFormat Format {
             get { return format; }
             set { format = value; }
         }
-        
-        protected override string Text {
-            get {
-                if (BoundObject == null)
-                    return String.Empty;
 
-                DateTime dt = (DateTime) BoundObject;
-                
-                if (dt == DateTime.MinValue)
-                    return String.Empty;
-
-                switch (Format)
-                {
-                case DateTimeFormat.Long:         return dt.ToString ();
-                case DateTimeFormat.ShortDate:    return dt.ToShortDateString ();
-                case DateTimeFormat.LongDate:     return dt.ToLongDateString ();
-                case DateTimeFormat.ShortTime:    return dt.ToShortTimeString ();
-                case DateTimeFormat.LongTime:     return dt.ToLongTimeString ();
-                }
-                
+        protected override string GetText (object obj)
+        {
+            if (obj == null) {
                 return String.Empty;
             }
+
+            DateTime dt = (DateTime) obj;
+
+            if (dt == DateTime.MinValue) {
+                return String.Empty;
+            }
+
+            switch (Format) {
+            case DateTimeFormat.Long:         return dt.ToString ();
+            case DateTimeFormat.ShortDate:    return dt.ToShortDateString ();
+            case DateTimeFormat.LongDate:     return dt.ToLongDateString ();
+            case DateTimeFormat.ShortTime:    return dt.ToShortTimeString ();
+            case DateTimeFormat.LongTime:     return dt.ToLongTimeString ();
+            }
+
+            return String.Empty;
         }
     }
 }
