@@ -5,12 +5,15 @@ using System.Collections.Generic;
 using Mono.Unix;
 using Gtk;
 
+using Hyena;
+
 using Banshee.Widgets;
 using Banshee.Sources;
 using Banshee.ServiceStack;
 using Banshee.Collection;
 using Banshee.Collection.Gui;
 using Banshee.Gui;
+using Banshee.Gui.Widgets;
 using Banshee.Sources.Gui;
 using Banshee.Web;
 
@@ -30,11 +33,6 @@ namespace Banshee.Lastfm.Radio
 
         private Viewport viewport;
 
-        static LastfmSourceContents () {
-            DataCore.UserAgent = Banshee.Web.Browser.UserAgent;
-            DataCore.CachePath = System.IO.Path.Combine (Banshee.Base.Paths.ExtensionCacheRoot, "lastfm");
-        }
-
         // "Coming Soon: Profile, Friends, Events etc")
         public LastfmSourceContents () : base ()
         {
@@ -51,11 +49,12 @@ namespace Banshee.Lastfm.Radio
 
             // Clamp the width, preventing horizontal scrolling
             SizeAllocated += delegate (object o, SizeAllocatedArgs args) {
-                main_box.WidthRequest = args.Allocation.Width - 10;
+                // TODO '- 10' worked for Nereid, but not for Cubano; properly calculate the right width we should request
+                main_box.WidthRequest = args.Allocation.Width - 30;
             };
 
             viewport.Add (main_box);
-            
+
             StyleSet += delegate {
                 viewport.ModifyBg (StateType.Normal, Style.Base (StateType.Normal));
                 viewport.ModifyFg (StateType.Normal, Style.Text (StateType.Normal));
@@ -74,7 +73,7 @@ namespace Banshee.Lastfm.Radio
 
             if (lastfm.Connection.Connected) {
                 UpdateForUser (lastfm.Account.UserName);
-            } else { 
+            } else {
                 lastfm.Connection.StateChanged += HandleConnectionStateChanged;
             }
 
@@ -137,14 +136,14 @@ namespace Banshee.Lastfm.Radio
             recently_loved.SetList (user.RecentLovedTracks);
             recently_played.SetList (user.RecentTracks);
             top_artists.SetList (user.GetTopArtists (TopType.Overall));
-            
+
             ShowAll ();
         }
 
         private void HandleConnectionStateChanged (object sender, ConnectionStateChangedArgs args)
         {
             if (args.State == ConnectionState.Connected) {
-                Banshee.Base.ThreadAssist.ProxyToMain (delegate {
+                ThreadAssist.ProxyToMain (delegate {
                     if (lastfm != null && lastfm.Account != null) {
                         UpdateForUser (lastfm.Account.UserName);
                     }
@@ -185,6 +184,7 @@ namespace Banshee.Lastfm.Radio
             public NumberedList (LastfmSource lastfm, string name) : base (name)
             {
                 this.lastfm = lastfm;
+                artwork_manager.AddCachedSize(40);
                 tile_view = new NumberedTileView (1);
                 PackStart (tile_view, true, true, 0);
                 tile_view.Show ();
@@ -226,7 +226,7 @@ namespace Banshee.Lastfm.Radio
                         AlbumInfo album = new AlbumInfo (track.Album);
                         album.ArtistName = track.Artist;
 
-                        Gdk.Pixbuf pb = artwork_manager == null ? null : artwork_manager.LookupScale (album.ArtworkId, 40);
+                        Gdk.Pixbuf pb = artwork_manager == null ? null : artwork_manager.LookupScalePixbuf (album.ArtworkId, 40);
                         if (pb != null) {
                             tile.Pixbuf = pb;
                         }
@@ -258,7 +258,7 @@ namespace Banshee.Lastfm.Radio
                 //menu.Append (new MenuItem ("Listen to Loved Station"));
                 //menu.Append (new MenuItem ("Listen to Neighbors Station"));
 
-                menu.ShowAll (); 
+                menu.ShowAll ();
                 menu.Popup (null, null, null, 0, Gtk.Global.CurrentEventTime);
                 menu.Deactivated += delegate {
                     (sender as Button).Relief = ReliefStyle.None;
@@ -267,36 +267,5 @@ namespace Banshee.Lastfm.Radio
         }
     }
 
-    public class TitledList : VBox
-    {
-        private Label title;
 
-        public string Title {
-            get { return title.Text; }
-            set {
-                title.Markup = String.Format ("<b>{0}</b>", GLib.Markup.EscapeText (value));
-            }
-        }
-
-        public int TitleWidthChars {
-            get { return title.WidthChars; }
-            set { title.WidthChars = value; }
-        }
-
-        public TitledList (string title_str) : base (false, 3)
-        {
-            title = new Label ();
-            title.Xalign = 0;
-            title.Ellipsize = Pango.EllipsizeMode.End;
-            Title = title_str;
-
-            PackStart (title, false, false, 0);
-            title.Show ();
-
-            StyleSet += delegate {
-                title.ModifyBg (StateType.Normal, Style.Base (StateType.Normal));
-                title.ModifyFg (StateType.Normal, Style.Text (StateType.Normal));
-            };
-        }
-    }
 }
