@@ -32,6 +32,8 @@ using System.Collections.Generic;
 using Mono.Unix;
 using Gtk;
 
+using Hyena;
+using Hyena.Widgets;
 using Migo.Syndication;
 
 using Banshee.Base;
@@ -57,9 +59,12 @@ namespace Banshee.Podcasting.Gui
     {
         private uint actions_id;
         private DatabaseSource last_source;
+        private PodcastSource podcast_source;
 
         public PodcastActions (PodcastSource source) : base (ServiceManager.Get<InterfaceActionService> (), "Podcast")
         {
+            this.podcast_source = source;
+
             AddImportant (
                 new ActionEntry (
                     "PodcastUpdateAllAction", Stock.Refresh,
@@ -164,6 +169,7 @@ namespace Banshee.Podcasting.Gui
 
         public override void Dispose ()
         {
+            ServiceManager.SourceManager.ActiveSourceChanged -= HandleActiveSourceChanged;
             Actions.UIManager.RemoveUi (actions_id);
             Actions.RemoveActionGroup (this);
             base.Dispose ();
@@ -198,7 +204,7 @@ namespace Banshee.Podcasting.Gui
 
         private void OnSelectionChanged (object o, EventArgs args)
         {
-            Banshee.Base.ThreadAssist.ProxyToMain (delegate {
+            ThreadAssist.ProxyToMain (delegate {
                 UpdateFeedActions ();
                 UpdateItemActions ();
             });
@@ -481,7 +487,7 @@ namespace Banshee.Podcasting.Gui
         {
             Feed feed = ActiveFeedModel.FocusedItem;
             if (feed != null) {
-                new PodcastFeedPropertiesDialog (feed).Run ();
+                new PodcastFeedPropertiesDialog (podcast_source, feed).Run ();
             }
         }
 
